import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
import scipy.signal as signal
import matplotlib as mpl

mpl.rc('xtick', labelsize=16)
mpl.rc('ytick', labelsize=16)
mpl.rc("axes",labelsize=18)

n_taps=129 # number of taps. Must be odd for type I filters (optimum for general purpose filters)
f_cut=0.1 # ratio to nyquist freq

# creates a FIR low pass filter
# possible windows to test: boxcar (rectangular) hamming, blackman, nuttall, hann

bz=signal.firwin(n_taps, f_cut, width=None, window='blackman', pass_zero=True, scale=True, nyq=1.0)

# prepare e denominator of the same order as the numerator but
# with all taps set to zero except the first one (zero order)
# this is necessary for the discrete time step and impulse response functions
# that require a denominator with a degree higher or equal to that of the numerator 

az=np.zeros(n_taps)
az[0]=1

# calculates and plots the frequency response
wz,pz=signal.freqz(bz,az)

plt.loglog(wz/(2*np.pi),abs(pz))
plt.show()

# calculates and plots the step response
tz,yz=signal.dstep([bz,az,1],n=n_taps) # Step response: requires deg(bz) <= deg(az)
plt.plot(tz,yz[0])
plt.show()

