import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
import scipy.signal as signal
import matplotlib as mpl

mpl.rc('xtick', labelsize=16)
mpl.rc('ytick', labelsize=16)
mpl.rc("axes",labelsize=18)

# Comparison between the frequency response of the bilinear, simple and ideal integrators
# Sampling time is supposed to be 1 s
# A few wornings are produced due to divide by zero exceptions

bz=1;                     # Numerator of the simple integrator
bz2=np.array([1.0,1.0])/2 # Numerator of the bilinear integrator 
az=np.array([1.0,-1.0])   # Denominator of both simple and bilinear integrator
wz,pz=signal.freqz(bz,az) # Frequency response of the simple integrator
wz2,pz2=signal.freqz(bz2,az) # Frequency response of the bilinear integrator

# plot the two frequency responses and compare with time continuous
# note that the frequency axis is normalized with respect to the sampling frequency

plt.loglog(wz/(2*np.pi),abs(pz),label="simple")

plt.loglog(wz2/(2*np.pi),abs(pz2),label="bilinear")

plt.loglog(wz/(2*np.pi),1.0/wz,label="Time continuous")
plt.legend()
plt.show()





