# -*- coding: cp1252 -*-
import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
import scipy.signal as signal
import matplotlib as mpl

mpl.rc('xtick', labelsize=16)
mpl.rc('ytick', labelsize=16)
mpl.rc("axes",labelsize=18)

# Simulate the discrete time approximation of a second order, low pass
# dynamic system ( massspringdamper )

# These are the parameters of the 2nd order system:
# w0 = characteristic frequency Q0: quality factor

Q0=3
w0=3.0

# This is  the sampling frequency of the discrete time system that gas to
# simulate the dynamic system.
# The higher fs, the better the approximation 

fs=10



# w0=2*fs*np.tan(w0/fs/2)
# w0=2*fs*np.arctan(w0/fs/2)


# Continuous time representation: b,a = numerator(s) , denominator(s)

a=np.array([1,w0/Q0,w0*w0])
b=np.array([w0*w0,])

# Freq. response and impulse response of the CT system
w,p=signal.freqs(b,a,worN=np.logspace(-1,2,500))
t,y=signal.impulse((b,a),T=np.linspace(0,20,500))

# Synthesis of the Discrete Time system from the CT model
# through bilinear transformation
# Set warp=1 to apply pre-warping before DT synthesis
# Note: the original CT system response is unchanged

warp=0

if warp==1:   
    w0_w=2*fs*np.tan(w0/fs/2)
    a_w=np.array([1,w0_w/Q0,w0_w*w0_w])
    b_w=np.array([w0_w*w0_w,])
    bz,az=signal.bilinear(b_w,a_w,fs)
else:
    bz,az=signal.bilinear(b,a,fs)
    




# Freq. response and impulse response of the DT system

wz,pz=signal.freqz(bz,az)
t,y=signal.impulse((b,a),T=np.linspace(0,20,500))


plt.loglog(w,abs(p),label="TC")
plt.loglog(wz*fs,abs(pz),label="DT")
plt.legend()
plt.show()



tz,yz=signal.dimpulse([bz,az,1.0/fs],n=20*fs)

plt.plot(t,y,label="CT")
plt.plot(tz,fs*yz[0],label="DT")
plt.legend()

plt.show()
