import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
import scipy.signal as signal
import matplotlib as mpl

mpl.rc('xtick', labelsize=16)
mpl.rc('ytick', labelsize=16)
mpl.rc("axes",labelsize=18)


# Syntheis of first order DT systems starting from a CT system
# to illustrate the function "cont2discrete" that allows more
# flexibility with respect to the function "bilinear", being
# possible to specify different methods. The more interesting are:

# bilinear (identical to the function bilinear)
# zoh : (default) zero order hold (optimum time domain approximation)
# euler : same as simple integrator substitution

f0=3
fs=10.0
mtd="zoh" # method

w0=2*np.pi*f0
T=1/fs

# First order low pass filter: b=numerator(s), a=denominator(s)

a=np.array([1,w0]) # a[0]s + a[1]
b=np.array([w0,]) # b[0] 


w,p=signal.freqs(b,a,worN=np.logspace(-2,2,500)*w0) # Freq response
t,y=signal.step((b,a),T=np.linspace(0,5,100)) # Step response

# synthesis using the cont2discrete function
# note that the output is in anusual format (maybe due to a bug),
# so that the operation shown after the function call is necessary to get bz

sist=signal.cont2discrete((b,a),T,method=mtd)
bz=sist[0][0]
az=sist[1]


wz,pz=signal.freqz(bz,az) # Freq response
tz,yz=signal.dstep([bz,az,T],n=50) # Step response


plt.loglog(w/2/np.pi,abs(p),label="CT")
plt.loglog(wz/2/np.pi*fs,abs(pz),label=mtd)

plt.legend(bbox_to_anchor=(0.3, 1))
plt.show()

plt.plot(t,y,label="CT")
plt.plot(tz,yz[0],label=mtd)

plt.legend(loc=4)

plt.show()
